"use client";
import { useId, useEffect, useMemo, useState, memo } from "react";

interface Props {
	content: {title: string, description: string}[],
	progresses: { [key: number]: number }
};

const Desktop = memo(function Desktop({ content, progresses }: Props ) {
	return (
		<div className="grid grid-cols-3 justify-between">
			{
				content.map((v, k)=>{
					return <div key={k}>
						<img
							src={`https://ik.imagekit.io/choreless/v2/images/home/how_it_works_${k+1}?tr=w-400`}
							alt="image"
							loading="lazy"
							className='h-[12.5rem] rounded-[10px] mt-5 object-cover ms-0.25'
						/>
						<div className="grid grid-cols-[auto_1fr] items-center mt-[13px]">
							<p
								className="w-[34px] h-[34px] rounded-full flex items-center justify-center text-2xl text-white font-bold"
								style={{backgroundImage: `linear-gradient(90deg, #3063FF 0%, #6087FF ${Math.min(progresses[k], 20)*5}%, #d9d9d9 ${(Math.min(progresses[k], 20))*5}%)` }}
							>
								{k+1}
							</p>
							<span
								className="h-[3px] w-full mx-auto"
								style={{
									background: `linear-gradient(90deg, #3063FF ${((progresses[k]-20)*100)/80}%, #22222220 ${((progresses[k]-20)*100)/80}%)`,
								}}
							/>
						</div>
						<div className='mt-3 me-16 text-slate-900'>
							<h2 className="text-2xl font-bold leading-loose">{v.title}</h2>
							<p>{v.description}</p>
						</div>
					</div>
				})
			}
		</div>
	);
});


const Phone = memo(function Phone({ content, progresses }: Props) {
	return (
		<>
			{
				content.map((v, k)=>{
					return <div key={k} className="grid grid-cols-[auto_1fr] gap-x-2.5">
						<div className="grid grid-rows-[auto_1fr]">
							<p
								className="w-[34px] h-[34px] rounded-full flex items-center justify-center text-2xl text-white font-bold"
								style={{backgroundImage: `linear-gradient(180deg, #3063FF 0%, #6087FF ${Math.min(progresses[k], 20)*5}%, #d9d9d9 ${(Math.min(progresses[k], 20))*5}%)` }}
							>
								{k+1}
							</p>
							<span
								className="w-[3px] h-full mx-auto"
								style={{
									background: `linear-gradient(180deg, #3063FF ${((progresses[k]-20)*100)/80}%, #22222220 ${((progresses[k]-20)*100)/80}%)`,
								}}
							/>
						</div>
						<div className="text-slate-900">
							<h2 className="text-2xl font-bold -mt-0.5">{v.title}</h2>
							<p>{v.description}</p>
							<img
									src={`https://ik.imagekit.io/choreless/v2/images/home/how_it_works_${1}?tr=w-400`}
									alt="image"
									loading="lazy"
									className='h-[12.5rem] rounded-[10px] object-cover ms-0.25 mt-4 mb-9'
								/>
						</div>
					</div>
				})
			}
		</>
	);
});

const segmentCalculate = (completed: number, subtract: number) => Math.min(Math.ceil(Math.min(Math.max(completed - subtract, 0), 25) * 4),100);

export default function Component() {
	const section_id = useId();
	const [screen_width, setScreenWidth] = useState(0);
	const [scroll_progress, setScrollProgress] = useState(0);

	const progresses = useMemo(() => {
		if (scroll_progress > 66) return { 0: 100, 1: 100, 2: segmentCalculate(scroll_progress, 66) };
		if (scroll_progress > 33) return { 0: 100, 1: segmentCalculate(scroll_progress, 33), 2: 0 };
		return { 0: segmentCalculate(scroll_progress, 0), 1: 0, 2: 0 };
	}, [scroll_progress]);

	function scroll() {
		setScreenWidth(innerWidth);

		let progress = 0;
		const rect = (document.getElementById(section_id) as HTMLElement).getBoundingClientRect();
		if (innerWidth < 992) progress = ((innerHeight - rect.top) * 100) / rect.height - 25;
		else progress = ((innerHeight - rect.bottom) * 100) / (innerHeight - rect.height) - 10;
		setScrollProgress(progress);
	}

	useEffect(() => {
		setScreenWidth(innerWidth);
		addEventListener("resize", scroll);
		addEventListener("scroll", scroll);
		return () => {
			removeEventListener("resize", scroll);
			removeEventListener("scroll", scroll);
		};
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	const content = [
		{ title: 'Easy ordering', description: 'Create an account and schedule a pickup from home any day of the week, via phone, sms, or online.' },
		{ title: 'Fast pickup and delivery', description: "Fill our Duffel bags or a 13-gallon kitchen bag with your laundry and place it close to your front door. Don't worry, we'll text you a reminder." },
		{ title: 'Enjoy', description: 'Your freshly folded laundry will be returned to you in 24, 48, or 72 hours, ensuring that your clothes arrive when you need them.' }
	]

	return (
		<section className="px-2.5 xl:mx-16 mt-[50px] mb-[100px]" id={section_id}>
			<div className="max-w-[1240px] mx-auto">
				<h1 className="text-slate-900 text-[32px] [@media(min-width:992px)]:text-5xl font-bold leading-10 text-center">How it works</h1>
				{
					(screen_width > 0 && screen_width < 992) ?
						<div className="mt-6"><Phone progresses={progresses} content={content} /></div>
					:
					(screen_width > 0 && <div className="mt-10"><Desktop progresses={progresses} content={content} /></div>)
				}
			</div>
		</section>
	);
}
